﻿/**
 * Copyright(c) Live2D Inc. All rights reserved.
 *
 * Use of this source code is governed by the Live2D Open Software license
 * that can be found at https://www.live2d.com/eula/live2d-open-software-license-agreement_en.html.
 */

#pragma once

#include "CubismFramework.hpp"
#include "Type/csmVector.hpp"
#include "Type/csmRectF.hpp"
#include "Type/csmMap.hpp"
#include <float.h>
#include <gxm.h>

#define ALIGN(x, a)					(((x) + ((a) - 1)) & ~((a) - 1))
#define MAX(a, b)					(((a) > (b)) ? (a) : (b))

extern const SceGxmProgram _binary_VertShaderClearSrc_gxp_start;
extern const SceGxmProgram _binary_FragShaderClearSrc_gxp_start;

static const SceGxmProgram *VertShaderClearSrc = &_binary_VertShaderClearSrc_gxp_start;
static const SceGxmProgram *FragShaderClearSrc = &_binary_FragShaderClearSrc_gxp_start;

//------------ LIVE2D NAMESPACE ------------
namespace Live2D { namespace Cubism { namespace Framework { namespace Rendering {


/**
 * @brief  オフスクリーン描画用構造体
 */
class CubismOffscreenFrame_GXM
{
public:

    CubismOffscreenFrame_GXM();
    virtual ~CubismOffscreenFrame_GXM();

    /**
     * @brief   指定の描画ターゲットに向けて描画開始
     *
     * @param   restoreFBO        0以上の場合、EndDrawでこの値をglBindFramebufferする
     */
    void BeginDraw(SceGxmContext **immContext);

    /**
     * @brief   描画終了
     *
     */
    void EndDraw(SceGxmContext **immContext);

    void Clear(float r, float g, float b, float a, SceGxmContext **immContext);

    csmBool CreateOffscreenFrame(csmUint32 displayBufferWidth, csmUint32 displayBufferHeight, SceGxmShaderPatcher **shaderPatcher, SceGxmMultisampleMode *msaa);

    /**
     * @brief   CubismOffscreenFrameの削除
     */
    void DestroyOffscreenFrame();

    /**
     * @brief   カラーバッファメンバーへのアクセッサ
     */
    SceGxmTexture *GetColorBuffer();

    /**
     * @brief   バッファ幅取得
     */
    csmUint32 GetBufferWidth() const;

    /**
     * @brief   バッファ高さ取得
     */
    csmUint32 GetBufferHeight() const;

    /**
     * @brief   現在有効かどうか
     */
    csmBool IsValid() const;

private:

    // offscreen surface data and render target
    SceGxmDeviceMemInfo				*_offscreenColorBufferData;
    SceGxmColorSurface				*_offscreenColorSurface;
    SceGxmTexture					*_offscreenTexture;
    SceGxmDeviceMemInfo				*_offscreenDepthBufferData;
    SceGxmDepthStencilSurface		*_offscreenDepthSurface;
    SceGxmRenderTarget				*_offscreenRenderTarget;

    SceGxmDeviceMemInfo *_clearDataStorage;

    const SceGxmProgramParameter *_clearColorParam;
    SceGxmVertexProgram *_clearVertexProgram;
    SceGxmFragmentProgram *_clearFragmentProgram;

    SceGxmShaderPatcherId _clearVertexProgramId;
    SceGxmShaderPatcherId _clearFragmentProgramId;
    SceGxmShaderPatcher *_shaderPatcher;

    csmUint32   _bufferWidth;           ///< Create時に指定された幅
    csmUint32   _bufferHeight;          ///< Create時に指定された高さ
};


}}}}

//------------ LIVE2D NAMESPACE ------------
